package BK::Client;

=head1 NAME

BK::Client - class to support the client side of BK

=head1 SYNOPSIS

     require BK::Client;

=head1 DESCRIPTION


=cut

require 5.002;
use BK::Composer;
use POSIX;

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self  = {
        EXITVAL     => EXIT_SUCCESS,
	FD          => undef,
        EDITOR      => $ENV{'EDITOR'} || "/bin/vi",
        _RELEASED   => 0,
    };
    bless ($self, $class);
    return $self;
}

sub fd {
    my $self = shift;
    if (@_) {
	$self->{FD} = shift;
        if (defined $self->{FD}) {
            my $old  = select($self->{FD});
            $| = 1;
            select($old);
        }
    }
    return $self->{FD};
}

sub editor {
    my $self = shift;
    if (@_) { $self->{EDITOR} = shift }
    return $self->{EDITOR};
}

sub exitval {
    my $self = shift;
    if (@_) { $self->{EXITVAL} = shift }
    return $self->{EXITVAL};
}

sub run {
    my ($self, $args) = @_;
    my $func = shift @$args;
    
    my $fd = $self->fd();

    print $fd &generate({
		func	=> "user-" . $func,
        }, $args);

    while (!$self->{_RELEASED}) {
        ($table, $args) = &ungenerate($fd);
        $self->respond($table, $args);
    }
     
    $self->fd(undef);   
}

sub respond {
    my ($self, $table, $args) = @_;

    (my $func = $table->{func}) =~ s/-//g;
    # print "func = \"$table->{func}\"\n";
    &$func($self, $table, $args);
}

# The client release function.

sub release {
    my $self = shift;
    $self->{_RELEASED} = 1;

    print {$self->fd()} &generate({
		func	=> "release",
        });
}


# Client routines
        
sub clientOutputData {
    my ($self, $table) = @_;

    print $table->{'data'};
}

sub clientReadData {
    my ($self, $table) = @_;

    print {$self->fd()} &generate({
                data    => join('', <STDIN>),
		func	=> $table->{confirm},
        });
}

sub clientEditData {
    my ($self, $table) = @_;

    my $tmpfile = POSIX::tmpnam();
    open(TMPFILE, ">$tmpfile");
    print TMPFILE $table->{data};
    close(TMPFILE);

    my $fail = system $self->editor(), $tmpfile;
    my @data = "";
    if (not $fail) {
        open(TMPFILE, "<$tmpfile");
        @data = <TMPFILE>;
        close(TMPFILE);
    }
    unlink $tmpfile;
        
    my $fd = $self->fd();
    print {$self->fd()} &generate({
                data    => join('', @data),
		func	=> $fail ? $table->{decline} : $table->{confirm},
        });
}

sub clientOutputInfo {
    my ($self, $table) = @_;

    print $table->{'data'}, "\n";
}

sub clientOutputError {
    my ($self, $table) = @_;

    print $table->{'data'}, "\n";
    $self->exitval(EXIT_FAILURE);
}

1;
