package BK::DateManager;

=head1 NAME

BK::Doc::Invoice - class to compose an invoice given a bunch of info

=head1 SYNOPSIS

     use BK::DateManager;

=head1 DESCRIPTION


=cut

require 5.002;
use POSIX;
use Time::Local;

sub new {
    my ($proto, $date) = @_;
    my $class = ref($proto) || $proto;
    my $self  = {
        VALUES     => {},
    };
    bless ($self, $class);
    $self->assignDates($date);
    return $self;
}

sub daysInMonth {
    my ($self, $mon, $year) = @_;

    if ($mon == 4 || $mon == 6 || $mon == 9 || $mon == 11) { "30" }
    elsif ($mon == 2 && $values{'%y'} % 4)                 { "28" }
    elsif ($mon == 2)                                      { "29" }
    else                                                   { "31" }
}

sub assignDates {
    my ($self, $date) = @_;
    $date = '' if (not defined $date);
    my($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) =
	localtime($date =~ /^(\d{4})\.(\d{2})(\d{2})$/
			? timelocal(0, 0, 0, $3, $2 -1, $1)
			: time);
    my $fdoy = 6 - (5 + $yday -$wday) % 7;

    my %values;
    $values{'%%'} = '%';
    $values{'%U'} = sprintf "%02d", int(($yday +$fdoy -1)/7);
    $values{'%d'} = sprintf "%02d", $mday;
    $values{'%m'} = sprintf "%02d", ++$mon;
    $values{'%Y'} = 1900 + $year;
    $values{'%y'} = sprintf "%02d", $year % 100;
    $values{'%j'} = sprintf "%03d", $yday;
    $values{'%l'} = &daysInMonth($mon, $year);
    %{ $self->{VALUES} } = %values;
}

sub makeDate {
    my ($self, @str) = (shift, split('', shift));
    my $result = '';
    
    while (@str) {
	my $char = shift @str;
	if ($char eq '%' and @str) {
	    $char .= shift @str;
	    if (defined $self->{VALUES}->{$char}) {
	        $result .= $self->{VALUES}->{$char};
		next;
	    }
	}
	$result .= $char;
    }
    $result;
}

sub incrDate {
    my ($self, $date, $dinc, $minc, $yinc) = @_;
    $dinc = 0 if (not $dinc);
    $minc = 0 if (not $minc);
    $yinc = 0 if (not $yinc);

    die "\"$date\" is not a valid date."
                if (!($date =~ /^(\d{4})\.(\d{2})(\d{2})$/));
    my ($y, $m, $d) = ($1, $2, $3);

    die "bad month \"$m\"." if ($m < 1 || $m > 12);
    die "bad day \"$d\"."   if ($d < 1 || $d > 31);
    die "bad year \"$y\"."  if ($y < 1900);

    $y += $yinc;

    $m += $minc;
    while ($m <= 0) {$y--; $m += 12;}
    while ($m > 12) {$y++; $m -= 12;}

    $d += $dinc;
    while ($d <= 0) {
	$m--; if ($m == 0) {$m = 12; $y--;}
	$d += $self->daysInMonth($m, $y);
    }
    while ($d > $self->daysInMonth($m, $y)) {
	$d -= $self->daysInMonth($m, $y);
	$m++; if ($m == 13) {$m = 1; $y++;}
    }
    sprintf "%04d.%02d%02d", $y, $m, $d;
}

1;
