package BK::Document;

=head1 NAME

BK::Doc::Invoice - class to compose an invoice given a bunch of info

=head1 SYNOPSIS

     use BK::Document;

=head1 DESCRIPTION


=cut

require 5.002;
use POSIX;

sub new {
    my ($proto, $home, $docclass, $format) = @_;
    my $class = ref($proto) || $proto;
    die "Unknown document format \"$format\""
	if ($format !~ /postscript/i && $format !~ /text/i);
    my $self  = {
	HOME     => $home,
	CLASS    => $docclass,
	FORMAT   => $format,
        INFO     => {},
	FIELDS   => undef,
	ITEMS    => [],
    };
    bless ($self, $class);
    return $self;
}

sub itemFields {
    my ($self) = shift;
    @{ $self->{FIELDS} } = @_;
}

sub addInfo {
    my ($self, $field, $value) = @_;
    # KLUGE fix the ugly loop in Print.pm.
    # Only take the first none empty setting.
    if (defined($self->{INFO}->{$field})) {
	return if ($self->{INFO}->{$field} ne '');
    }
    $self->{INFO}->{$field} = $value;
}

sub addItem {
    my ($self, $item) = @_;
    push @{ $self->{ITEMS}} , $item;
}

sub dumpInfo {
    my ($self) = @_;

    print "dump: home = $self->{HOME}\n";
    print "dump: format = $self->{FORMAT}\n";
    while (($key, $value) = each %{ $self->{INFO} }) {
	print "dump: $key = $value\n";
    }

    print "dump: ";
    foreach $field (@{$self->{FIELDS}}) {
	print "$field	";
    }
    print "\n";
    foreach $item (@{$self->{ITEMS}}) {
	print "dump: $item\n";
    }
}
	
sub generate() {
    my ($self) = @_;

    if ($self->{FORMAT} =~ /postscript/i) {
    	return $self->generatePostScript();
    }
    if ($self->{FORMAT} =~ /text/i) {
    	return $self->generateText();
    }
}

sub generateText {
    my ($self) = @_;
    my $result = "";
    while (($key, $value) = each %{ $self->{INFO} }) {
	$result .= "$key = $value\n";
    }
    $result .= join("\t", @{$self->{FIELDS}}) . "\n";
    foreach (@{$self->{ITEMS}}) {
	$result .= "$_\n";
    }
    $result;
}
	
sub generatePostScript {
    my ($self) = @_;
    my $result = "";
    my $date = ctime(time);
    chop $date;

    $result .= <<__EOH__ ;
%!PS-Adobe-3.0 EPSF-3.0
%%Creator: BK
%%For: $ENV{USER}
%%Title: $self->{CLASS}
%%CreationDate: $date
%%Origin: 0 0
%%BoundingBox: 0 0 612 792
%%Pages: 1
%%Orientation: Portrait
__EOH__

    my $prolog = $self->{HOME} . "/common.ps";
    open (PROLOG, "<$prolog") || die "Can't open $prolog: $!\n";
    while (<PROLOG>) {
	$result .= $_;
    }
    close PROLOG;

    $prolog = $self->{HOME} . "/" . $self->{CLASS} . ".ps";
    open (PROLOG, "<$prolog") || die "Can't open $prolog: $!\n";
    while (<PROLOG>) {
	$result .= $_;
    }
    close PROLOG;


    $result .= <<__EOH__ ;
%%BeginSetup
/setstrokeadjust where {pop true setstrokeadjust} if
%%EndSetup
%%Page: 1 1
save

__EOH__


    # Do the Invoice Header.
    while (($key, $value) = each %{ $self->{INFO} }) {
	$result .= "/$key ($value) def\n";
    }
    $result .= "DoHeader\n\n";

    # Now do each item.
    foreach $item (@{$self->{ITEMS}}) {
	@values = split /\t/, $item;

	foreach $field (@{ $self->{FIELDS} }) {
            my $value = shift(@values);
            if (not defined $value) { $value = ""; }
	    $result .= "/$field (" . $value . ") def\n";
	}

        $result .= "DoItem\n\n";
    }
    $result .= "DoTrailer\n";

    $result .= <<__EOH__ ;
restore
showpage

%%Trailer
end
%%EOF
__EOH__

    $result;
}

1;
