package BK::ServFunc::Help;

require 5.002;
use BK::Composer;
use POSIX;

sub register {
    return {
	"user-help"	=> { func => \&BK::ServFunc::Help::help,
			     shorthelp => "Returns this message",
			   },
    };
}

sub help {
    my ($server, $table, $args) = @_;
    my $fd = $server->fd();
    
    if (not scalar(@$args)) {
        print $fd &generate({data => <<'__EOH__',func => 'client-OutputInfo'});

    BANAL -- FTL bookkeeping system

    bk is BANAL's client tool for the command line.  Try:

       bk help commands       for a list of commands
       bk help command        for help on a specific command
       bk help usage          for generic command line arguments
__EOH__
    } else {
	foreach  $topic (@$args) {
            my ($help, $func) = clientHelp($server, $topic);

	    if (defined $server->{COMMANDS}->{"user-" . $topic} &&
	        defined $server->{COMMANDS}->{"user-" . $topic}->{'help'}) {
	        $help = $server->{COMMANDS}->{"user-" . $topic}->{'help'};
                $func = 'client-OutputInfo';
	    }

            print $fd &generate({
                            data => $help,
                            func => $func,
                       });
	}
    }
    $server->release();
}

sub clientHelp {
    my ($server, $topic) = @_;

    $_ = $topic;
    return do {
        if (/commands/) {
            my $help = "\n    BANAL client commands:\n";
            foreach (sort keys %{$server->{COMMANDS}}) {
                (/^user-(.*)/) && do {
                    $help .= sprintf "\n        %-12s %s",
                    $1,
                    $server->{COMMANDS}->{$_}->{'shorthelp'};
                };
            }
            ($help, 'client-OutputInfo')}
        elsif (/usage/) {(<<'__EOH__', 'client-OutputInfo')}

    BANAL client usage:

    bk [ -L dir -p port ] command [ arg ... ]

    The -L flag will set the library directory.  It will default to a
    sane location based on the program's location.

    The -p flag specifies the server's listen address, overriding the
    value of $BKPORT in the environment and the default (localhost:1678).
__EOH__
        else            {("no help available for \"$_\"",
                          'client-OutputError')}
    };
}

1;
