package BK::ServFunc::ListFunc;

require 5.002;
use BK::Composer;

sub register {
    my ($books) = shift;

    opendir(BOOKS, $books) || die "Can't open $books: $!";
    my %cmds;
    foreach $db (map { chop;chop;chop;chop; "\l$_"; }
                        grep {/.*\.rdb/} readdir(BOOKS)) {
        # print "loading: $db\n";
	# pluralize it (anything ending with 'y' get 'ies')
        my $plural = $db;
        if (substr($plural, -1, 1) ne 'y') {
	    $plural .= 's';
        } else {
	    $plural =~ s/y$/ies/;
	}

	# Make the list functions.
	my $cmd = BK::ServFunc::ListFunc::newFunc($db, $plural);

	$cmds{"user-$plural"} = $cmd;
    }
    if (not %cmds) {
        print "WARNING: unable to find any RDB tables.  ",
                "Consider specifying a directory.\n";
    }

    closedir(BOOKS);
    return \%cmds;
}

sub newFunc {
    my ($base, $plural) = @_;
    my $base3 = substr($base, 0, 3);
    my $a = 'a';

    $a = 'an' if $base =~ /^[aeiou]/;
    
    return { shorthelp => "Display a list of $plural",
	     func => sub { BK::ServFunc::ListFunc::list("\u$base", @_); },
	     help => <<"__EOH__", };

    $plural -- Display a list of $plural

    bk $plural [ -f SEP ] [ -r ] [ -c FIELDS ] [ expression ]
    bk $plural [ -f SEP ] [ -F ] [ -T ] [ -D ]

        Reports a list of all $plural currently known to the system.
        An expression may be any valid RDB expression.  The -f option
        specifies the field seperator (eg. -f : or -f XXX).  The default
        seperator is a tab space (<TAB>).

        Use the -r flag to generate an entire RDB file instead of just a
        listing of entries.

        The -c flag lets the user specify which fields (and which order) to
        report the entries.  FIELDS must be one single argument if more
        than one field is requested (ie. -c 'Field1 Field2')

        This command may also return the field names (-F), the field
        types (-T), and the Field descriptions (-D).
__EOH__
}

sub list {
    my ($name, $server, $table, $args) = @_;
    my ($cmd, $sep) = ('true', "\t");
    my ($keepHead, $showField, $showTypes, $showDescr, $fields) =
                 (0, 0, 0, 0, undef);

    if (defined $args) {
        while (defined $$args[0] && $$args[0] =~ /^-/) {
            my $opt = shift(@$args);
            $fields = shift(@$args), next if ($opt eq '-c');
            $sep    = shift(@$args), next if ($opt eq '-f');
            $keepHead  = 1, next          if ($opt eq '-r');
            $showField = 1, next          if ($opt eq '-F');
            $showTypes = 1, next          if ($opt eq '-T');
            $showDescr = 1, next          if ($opt eq '-D');
            die "Unknown option \"$opt\".  Try 'bk help \l$name'.";
        }
        die "no field seperator specified with '-f'" if not $sep;
        $cmd = "'" . join("' '", @$args) . "'" if ($#$args >= 0);
    }
    my @entries;
    if (!$showField && !$showTypes && !$showDescr) {
        @entries = $server->getEntries($name, $cmd, $fields, $keepHead);
    } else {
        my ($f, $t, $d) = $server->getHeader($name);
        push(@entries, join("\t", @$f)) if ($showField);
        push(@entries, join("\t", @$t)) if ($showTypes);
        push(@entries, join("\t", @$d)) if ($showDescr);
    }

    foreach $entry (map {s/\t/$sep/g if ($sep ne "\t"); $_ } @entries) {
        print {$server->fd()} &generate({
                      data => $entry,
                      func => 'client-OutputInfo',
                  });
    }
    $server->release();
}

1;
