package BK::ServFunc::Print;

require 5.002;
use BK::Composer;
require BK::Document;
use POSIX;

sub register {
    my $cmd = 'print';
    return {
	"user-$cmd"	=> { func      => \&BK::ServFunc::Print::genInvoice,
			     shorthelp => "Generate an invoice",
			     help      => "
    $cmd -- Generate an invoice

    bk $cmd [ -d doctype ] InvoiceID

        \u$cmd generates a file containing information from the
        InvoiceID and related Charges.

        If the invoice has no Date set then it defaults to today
        and this value is saved in the database.

        Use the -d option to specify a documant type.  Currently,
	the 'postscript' and 'text' are available.  Postscript is
        the default.
",
			   },
    };
}

sub genInvoice {
    my ($server, $table, $args) = @_;

    die "No Invoice ID specified" if not @$args;
    my ($ID, $docType) = (shift(@$args), 'postscript');
    if ($ID =~ /-d/) {
        die "No Invoice ID specified" if @$args != 2;
	($docType, $ID) = (shift(@$args), shift(@$args));
    }

    my $doc = BK::Document->new($server->libdir() . "/templates",
				    'Invoice',
				    $docType);
    my $db = 'Invoice';
    my $searchVal = $ID;

    # XXX This is ugly and still doesn't do the 'deep' copying correctly.
DUMPROW: {
    my @fields = @{($server->getHeader($db))[0]};
    my @entries = $server->getEntries($db, "$db eq '$searchVal'");
    my ($fieldNum, $updateDate) = (0, 0);

    die "Unable to find entry \"$searchVal\" in the $db table" if not @entries;
    undef $db; #reset
    my @info = split /\t/, shift(@entries);

    foreach $field (@fields) {
	$val = $info[$fieldNum];
	if (not defined $val) {$val = '';}
	if ($field eq 'Date' && $val eq '') {
	    my($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst);
	    ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) =
		localtime(time);

	    $year += 1900;
	    $val = sprintf "%4d.%02d%02d", $year, $mon +1, $mday;
            $updateDate = 1;
        }
	if ($fieldNum && ($field eq 'Client' || $field eq 'Company')) {
                $db = $field;
                $searchVal = $val;
	}
	$doc->addInfo($field, $val);

        if ($updateDate) {
            $info[$fieldNum] = $val;
            $server->reallyDelete('Invoice', $ID);
            my $entry = join "\t", @info;
            # print "Entry is $entry\n";

            my $dbname = $server->home() . "/" . 'Invoice' . ".rdb";

            # Untaint $dbname.
	    $dbname =~ /(.*)/; $dbname = $1;

            open(DATABASE, ">>$dbname") || die "Can't open $dbname: $!\n";
            print DATABASE $entry, "\n";
            close DATABASE;
            $updateDate = 0;
        }
        ++$fieldNum;
    }
    if (defined $db) { redo DUMPROW; }
}

    my @fields = @{($server->getHeader('Charge'))[0]};
    my @entries = $server->getEntries('Charge', "Invoice eq '$ID'");

    push @fields, 'ItemTotal';
    $doc->itemFields(@fields);

    # Find some key fields
    my ($num, $quanID, $costID, $taxID) = (0, undef, undef, undef);
    for (@fields) {
	$quanID = $num if /^Quantity$/;
	$costID = $num if /^UnitCost$/;
	$taxID = $num  if /^Tax$/;
	$num++;
    }
    my ($subtotal, $taxtotal, %taxtotal) = (0, 0, ());
    foreach $line (@entries) {
        my ($quan, $cost, $tax) = (split "\t", $line)[$quanID, $costID, $taxID];
	$cost *= $quan;
	$subtotal += $cost;
	my %taxes = $server->calcTaxes($cost, $tax);
	while (($key,$val) = each %taxes) {
	    $taxtotal{$key} += $val;
	    $taxtotal += $val;
        }
	$doc->addItem($line . "\t$cost");
    }
    # $doc->dumpInfo();
    while (($key,$val) = each %taxtotal) {
	$doc->addInfo($key, $val);
    }
    $doc->addInfo('Subtotal', $subtotal);
    $doc->addInfo('Total', $subtotal +$taxtotal);

    print {$server->fd()} &generate({
                     data => $doc->generate(),
                     func => 'client-OutputData',
               });
    $server->release();
}

1;
